/* texture.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999-2000 DindinX <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include "giram.h"
#include "utils.h"
#include "gtkcolorbutton.h"
#include "texture.h"

#include "giramwidgets.h"

#include "giramintl.h"

#define UPDATE_SENSITIVE(a, b, c)  do { \
 gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(Dialog, "First Color")), (a));        \
 gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(Dialog, "First Color Label")), (a));  \
 gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(Dialog, "Second Color")), (b));       \
 gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(Dialog, "Second Color Label")), (b)); \
 gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(Dialog, "Third Color")), (c));        \
 gtk_widget_set_sensitive(GTK_WIDGET(gtk_object_get_data(Dialog, "Third Color Label")), (c));  \
 } while (0)

static struct {
  GiramPigmentType type;
  gchar *label;
} pigment_type_list[] =
{
  { PAT_SOLID_COLOR, "Solid"},
  { PAT_BRICK,       "Brick"},
  { PAT_CHECKER,     "Checker"},
  { PAT_HEXAGON,     "Hexagon"},
  { PAT_LAST,        NULL}
};


/*****************************************************************************
*  ChangeColorDialogOK
******************************************************************************/
static void ChangeColorDialogOK(GtkWidget *Dialog)
{
  GtkWidget     *ColorButton;
  ViewStruct    *view_data;
  PigmentStruct *Pigment;
  GtkWidget     *entry;
  guint          i;
  ObjectStruct  *object;

  view_data = get_current_view_data();
  object = view_data->frame->selection->data;
  Pigment = object->Texture->Pigment;
  /* Save Type */
  entry = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(Dialog), "Entry"));
  for (i=0 ; pigment_type_list[i].label ; i++)
    if (!strcmp(pigment_type_list[i].label, gtk_entry_get_text(GTK_ENTRY(entry))))
    {
      Pigment->Type = pigment_type_list[i].type;
      break;
    }

  /* Save First Color */
  ColorButton = gtk_object_get_data(GTK_OBJECT(Dialog), "First Color");
  Pigment->Color[0] = GTK_COLOR_BUTTON(ColorButton)->color[0];
  Pigment->Color[1] = GTK_COLOR_BUTTON(ColorButton)->color[1];
  Pigment->Color[2] = GTK_COLOR_BUTTON(ColorButton)->color[2];
  Pigment->Color[3] = GTK_COLOR_BUTTON(ColorButton)->color[3];
  /* Save Second Color */
  ColorButton = gtk_object_get_data(GTK_OBJECT(Dialog), "Second Color");
  Pigment->Color2[0] = GTK_COLOR_BUTTON(ColorButton)->color[0];
  Pigment->Color2[1] = GTK_COLOR_BUTTON(ColorButton)->color[1];
  Pigment->Color2[2] = GTK_COLOR_BUTTON(ColorButton)->color[2];
  Pigment->Color2[3] = GTK_COLOR_BUTTON(ColorButton)->color[3];
  /* Save Third Color */
  ColorButton = gtk_object_get_data(GTK_OBJECT(Dialog), "Third Color");
  Pigment->Color3[0] = GTK_COLOR_BUTTON(ColorButton)->color[0];
  Pigment->Color3[1] = GTK_COLOR_BUTTON(ColorButton)->color[1];
  Pigment->Color3[2] = GTK_COLOR_BUTTON(ColorButton)->color[2];
  Pigment->Color3[3] = GTK_COLOR_BUTTON(ColorButton)->color[3];
/*  for (ObjectSel = UserData->FrameView->Selection ;
       ObjectSel->Next != UserData->FrameView->Selection ;
       ObjectSel = ObjectSel->Next)
  {
    gtk_color_selection_get_color(GTK_COLOR_SELECTION(ColorSel),
        ObjectSel->Object->Texture->Pigment->Color);
  }*/
  gtk_widget_destroy(Dialog);
}

/*****************************************************************************
* giram_texture_pigment_type_change
******************************************************************************/
static void giram_texture_pigment_type_change(GtkWidget *entry)
{
  guint             i;
  GiramPigmentType  new_type = PAT_LAST;
  GtkObject        *Dialog = GTK_OBJECT(gtk_widget_get_toplevel(entry));

  for (i=0 ; pigment_type_list[i].label ; i++)
    if (!strcmp(pigment_type_list[i].label, gtk_entry_get_text(GTK_ENTRY(entry))))
      new_type = pigment_type_list[i].type;
  switch (new_type)
  {
    case PAT_SOLID_COLOR:
      UPDATE_SENSITIVE(TRUE, FALSE, FALSE);
      break;
    case PAT_BRICK:
      UPDATE_SENSITIVE(TRUE, TRUE, FALSE);
      break;
    case PAT_CHECKER:
      UPDATE_SENSITIVE(TRUE, TRUE, FALSE);
      break;
    case PAT_HEXAGON:
      UPDATE_SENSITIVE(TRUE, TRUE, TRUE);
      break;
    case PAT_LAST:
      g_message("Error: unknown patern type\n");
      break;
  }
}

/*****************************************************************************
*  ChangeColorDialog
******************************************************************************/
void ChangeColorDialog(void)
{
  GtkWidget    *Dialog, *ColorButton, *Label, *Combo, *Table, *HBox;
  ViewStruct   *view_data;
  gdouble       color[4], color2[4], color3[4];
  GList        *TypeList = NULL;
  guint         i;
  ObjectStruct *object;

  view_data = get_current_view_data();
  if (!(view_data->frame->selection))
    return;

  object = view_data->frame->selection->data;

  Dialog = giram_dialog_new(_("Texture Editor"), "giram_message",
                            NULL /* FIXME */,
                            "dialog/texture_editor.html",
                            GTK_WIN_POS_MOUSE,
                            FALSE, FALSE, FALSE,
                            _("OK"), ChangeColorDialogOK,
                            NULL, 1, NULL, TRUE, FALSE,
                            NULL);

  HBox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(Dialog)->vbox), HBox);
  Label = gtk_label_new(_("Pigment type:"));
  gtk_box_pack_start_defaults(GTK_BOX(HBox), Label);
  /* The Pigment type combo box */
  Combo = gtk_combo_new();
  gtk_box_pack_start_defaults(GTK_BOX(HBox), Combo);
  for (i=0 ; pigment_type_list[i].label ; i++)
  {
    TypeList = g_list_append(TypeList, pigment_type_list[i].label);
  }
  gtk_combo_set_popdown_strings(GTK_COMBO(Combo), TypeList);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(Combo)->entry), FALSE);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(Combo)->entry), "changed",
                     (GtkSignalFunc)giram_texture_pigment_type_change, NULL);
  gtk_object_set_data(GTK_OBJECT(Dialog), "Entry", GTK_COMBO(Combo)->entry);

  Table = gtk_table_new(3, 2, FALSE);
  gtk_box_pack_start_defaults(GTK_BOX(HBox), Table);
  Label = gtk_label_new(_("Solid color"));
  gtk_object_set_data(GTK_OBJECT(Dialog), "First Color Label", Label);
  gtk_table_attach_defaults(GTK_TABLE(Table), Label, 0,1, 0,1);

  color[0] = object->Texture->Pigment->Color[0];
  color[1] = object->Texture->Pigment->Color[1];
  color[2] = object->Texture->Pigment->Color[2];
  color[3] = object->Texture->Pigment->Color[3];
  ColorButton = gtk_color_button_new("Pigment Color", 30, 20, color);
  gtk_table_attach_defaults(GTK_TABLE(Table), ColorButton, 1,2, 0,1);
  gtk_object_set_data(GTK_OBJECT(Dialog), "First Color", ColorButton);

  Label = gtk_label_new(_("Second color"));
  gtk_object_set_data(GTK_OBJECT(Dialog), "Second Color Label", Label);
  gtk_widget_set_sensitive(Label, FALSE);
  gtk_table_attach_defaults(GTK_TABLE(Table), Label, 0,1, 1,2);
  color2[0] = object->Texture->Pigment->Color2[0];
  color2[1] = object->Texture->Pigment->Color2[1];
  color2[2] = object->Texture->Pigment->Color2[2];
  color2[3] = object->Texture->Pigment->Color2[3];
  ColorButton = gtk_color_button_new("Pigment Color", 30, 20, color2);
  gtk_object_set_data(GTK_OBJECT(Dialog), "Second Color", ColorButton);
  gtk_widget_set_sensitive(ColorButton, FALSE);
  gtk_table_attach_defaults(GTK_TABLE(Table), ColorButton, 1,2, 1,2);
  Label = gtk_label_new(_("Third color"));
  gtk_object_set_data(GTK_OBJECT(Dialog), "Third Color Label", Label);
  gtk_widget_set_sensitive(Label, FALSE);
  gtk_table_attach_defaults(GTK_TABLE(Table), Label, 0,1, 2,3);
  color3[0] = object->Texture->Pigment->Color3[0];
  color3[1] = object->Texture->Pigment->Color3[1];
  color3[2] = object->Texture->Pigment->Color3[2];
  color3[3] = object->Texture->Pigment->Color3[3];
  ColorButton = gtk_color_button_new("Pigment Color", 30, 20, color3);
  gtk_object_set_data(GTK_OBJECT(Dialog), "Third Color", ColorButton);
  gtk_widget_set_sensitive(ColorButton, FALSE);
  gtk_table_attach_defaults(GTK_TABLE(Table), ColorButton, 1,2, 2,3);

  /* Dialog init */
  for (i=0 ; pigment_type_list[i].label ; i++)
    if (pigment_type_list[i].type == object->Texture->Pigment->Type)
    {
      gtk_list_select_item(GTK_LIST(GTK_COMBO(Combo)->list), i);
      break;
    }
  gtk_widget_show_all(Dialog);
}

